package org.example.binaryFollowMe;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import fr.liglab.adele.icasa.device.DeviceListener;
import fr.liglab.adele.icasa.device.GenericDevice;
import fr.liglab.adele.icasa.device.light.BinaryLight;
import fr.liglab.adele.icasa.device.presence.PresenceSensor;

public class BinaryFollowMeImpl implements DeviceListener<PresenceSensor> {

	/** Field for binaryLights dependency */
	private BinaryLight[] binaryLights;

	/** Field for presenceSensors dependency */
	private PresenceSensor[] presenceSensors;

	/**
	 * Bind Method for binaryLights dependency.
	 * This method is implemented for debug purpose only.
	 */
	public void bindBinaryLight(BinaryLight binaryLight, Map<Object, Object> properties) {
		System.out.println("bind binary light " + binaryLight.getSerialNumber());
	}

	/**
	 * Unbind Method for binaryLights dependency.
	 * This method is implemented for debug purpose only.
	 */
	public void unbindBinaryLight(BinaryLight binaryLight, Map<Object, Object> properties) {
		System.out.println("unbind binary light " + binaryLight.getSerialNumber());
	}

	/**
	 * Bind Method for PresenceSensors dependency.
	 * This method is used to manage device listener.
	 */
	public synchronized void bindPresenceSensor(PresenceSensor presenceSensor,
	        Map<Object, Object> properties) {
		// Add the listener to the presence sensor
		presenceSensor.addListener(this);
		System.out.println("bind presence sensor " + presenceSensor.getSerialNumber());

	}

	/**
	 * Unbind Method for PresenceSensors dependency.
	 * This method is used to manage device listener.
	 */
	public synchronized void unbindPresenceSensor(PresenceSensor presenceSensor,
	        Map<Object, Object> properties) {
		// Remove the listener from the presence sensor
		presenceSensor.removeListener(this);
		System.out.println("unbind presence sensor " + presenceSensor.getSerialNumber());
	}

	/**
	 * Component Lifecycle Method - this method is implemented for debug purpose
	 */
	public synchronized void stop() {
		System.out.println("Stopping light follow me");
		/*
		 * It is extremely important to unregister the device listener.
		 * Otherwise, iCASA will continue to send notifications to the
		 * unpredictable and invalid component instance.
		 * This will also cause problem when the bundle is stopped as iCASA
		 * will still hold a reference on the device listener object.
		 * Consequently, it (and its bundle) won't be garbage collected
		 * causing a memory issue known as stale reference.
		 */
		for (PresenceSensor presenceSensor : presenceSensors) {
			presenceSensor.removeListener(this);
		}
	}

	/**
	 * Component Lifecycle Method - this method is implemented for debug purpose
	 */
	public synchronized void start() {
		System.out.println("Starting light follow me");
	}

	/**
	 * Return all BinaryLight from the given location
	 * 
	 * @param location
	 *            : the given location
	 * @return the list of matching BinaryLights
	 */
	private synchronized List<BinaryLight> getBinaryLightFromLocation(String location) {
		List<BinaryLight> binaryLightsLocation = new ArrayList<BinaryLight>();
		for (BinaryLight binLight : binaryLights) {
			if (binLight.getPropertyValue(BinaryLight.LOCATION_PROPERTY_NAME).equals(location)) {
				binaryLightsLocation.add(binLight);
			}
		}
		return binaryLightsLocation;
	}

	/**
	 * This method is part of the DeviceListener interface and is called when a
	 * subscribed device property is modified.
	 * 
	 * @param device
	 *            is the device whose property has been modified.
	 * @param propertyName
	 *            is the name of the modified property.
	 * @param oldValue
	 *            is the old value of the property
	 */
	@Override
	public void devicePropertyModified(PresenceSensor changingSensor, String propertyName,
	        Object oldValue, Object newValue) {

		// check the change is related to presence sensing
		if (propertyName.equals(PresenceSensor.PRESENCE_SENSOR_SENSED_PRESENCE)) {
			// get the location where the sensor is:
			String detectorLocation = (String) changingSensor
			        .getPropertyValue(BinaryLight.LOCATION_PROPERTY_NAME);
			// if the location is known :
			if (!detectorLocation.equals(BinaryLight.LOCATION_UNKNOWN)) {
				// get the related binary lights
				List<BinaryLight> sameLocationLigths = getBinaryLightFromLocation(detectorLocation);
				for (BinaryLight binaryLight : sameLocationLigths) {
					// and switch them on/off depending on the sensed presence
					binaryLight.setPowerStatus(!(Boolean) oldValue);
				}
			}
		}

	}

	@Override
	public void deviceAdded(PresenceSensor sensor) {}

	@Override
	public void deviceEvent(PresenceSensor sensor, Object arg1) {}

	@Override
	public void devicePropertyAdded(PresenceSensor sensor, String arg1) {}

	@Override
	public void devicePropertyRemoved(PresenceSensor sensor, String arg1) {}

	@Override
	public void deviceRemoved(PresenceSensor sensor) {}

}
